# -*- coding: utf-8 -*-
"""
    pip_services3_mongodb.build.__init__
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

__all__ = ['DefaultMongoDbFactory']

from .DefaultMongoDbFactory import DefaultMongoDbFactory



# -*- coding: utf-8 -*-
from typing import Any


class MongoDbIndex:
    """
    Index definition for mongodb
    """

    def __init__(self, keys: Any, options: Any = None):
        # Index keys (fields)
        self.keys: Any = keys
        # Index options
        self.options: Any = options



# -*- coding: utf-8 -*-
"""
    pip_services3_mongodb.connect.__init__
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    MongoDbConnectionResolver module initialization.
    
    :copyright: Conceptual Vision Consulting LLC 2018-2019, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

__all__ = ['MongoDbConnectionResolver', 'MongoDbConnection']

from .MongoDbConnection import MongoDbConnection
from .MongoDbConnectionResolver import MongoDbConnectionResolver



# -*- coding: utf-8 -*-
"""
    test.persistence.DummyMongoDbPersistence
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Dummy MongoDb persistence implementation
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""
from typing import Optional

from pip_services3_commons.data import FilterParams, DataPage, PagingParams

from pip_services3_mongodb.persistence import IdentifiableMongoDbPersistence
from test.fixtures.IDummyPersistence import IDummyPersistence


class DummyMongoDbPersistence(IdentifiableMongoDbPersistence, IDummyPersistence):

    def __init__(self):
        super(DummyMongoDbPersistence, self).__init__("dummies")

    def _define_schema(self):
        self._ensure_index({'key': 1})

    def get_page_by_filter(self, correlation_id: Optional[str], filter: FilterParams, paging: PagingParams) -> DataPage:
        filter = filter or FilterParams()
        key = filter.get_as_nullable_string('key')

        filter_condition = {}
        if key is not None:
            filter_condition['key'] = key

        return super().get_page_by_filter(correlation_id, filter, paging, None, None)

    def get_count_by_filter(self, correlation_id: Optional[str], filter: FilterParams) -> int:
        filter = filter or FilterParams()
        key = filter.get_as_nullable_string('key')

        filter_condition = {}
        if key is not None:
            filter_condition['key'] = key

        return super().get_count_by_filter(correlation_id, filter)



# -*- coding: utf-8 -*-
"""
    pip_services3_mongodb.persistence.__init__
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    MongoDB module initialization
    
    :copyright: Conceptual Vision Consulting LLC 2018-2019, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

__all__ = ['MongoDbPersistence', 'IdentifiableMongoDbPersistence', 'MongoDbIndex']

from .IdentifiableMongoDbPersistence import IdentifiableMongoDbPersistence
from .MongoDbIndex import MongoDbIndex
from .MongoDbPersistence import MongoDbPersistence



# -*- coding: utf-8 -*-
"""
    test.IDummyPersistence
    ~~~~~~~~~~~~~~~~~~~~~~
    
    Interface for dummy persistence components
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""
from typing import Optional, List

from pip_services3_commons.data import FilterParams, PagingParams, DataPage, AnyValueMap
from pip_services3_data import IGetter, IWriter, IPartialUpdater

from test.fixtures.Dummy import Dummy


class IDummyPersistence(IGetter, IWriter, IPartialUpdater):

    def get_page_by_filter(self, correlation_id: Optional[str], filter: Optional[FilterParams], paging: Optional[PagingParams]) -> DataPage:
        raise NotImplementedError('Method from interface definition')

    def get_count_by_filter(self, correlation_id: Optional[str], filter: Optional[FilterParams]) -> int:
        raise NotImplementedError('Method from interface definition')

    def get_one_by_id(self, correlation_id: Optional[str], id: str) -> Dummy:
        raise NotImplementedError('Method from interface definition')

    def get_list_by_ids(self, correlation_id: Optional[str], ids: List[str]) -> List[Dummy]:
        raise NotImplementedError('Method from interface definition')

    def create(self, correlation_id: Optional[str], entity: Dummy) -> Dummy:
        raise NotImplementedError('Method from interface definition')

    def update(self, correlation_id: Optional[str], entity) -> Dummy:
        raise NotImplementedError('Method from interface definition')

    def update_partially(self, correlation_id: Optional[str], id: str, data: AnyValueMap) -> Dummy:
        raise NotImplementedError('Method from interface definition')

    def delete_by_id(self, correlation_id: Optional[str], id: str) -> Dummy:
        raise NotImplementedError('Method from interface definition')

    def delete_by_ids(self, correlation_id: Optional[str], ids: List[str]):
        raise NotImplementedError('Method from interface definition')



# -*- coding: utf-8 -*-
"""
    pip_services3_mongodb.persistence.MongoDbPersistence
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    MongoDb persistence implementation

    :copyright: Conceptual Vision Consulting LLC 2018-2019, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""
import random
import threading
from copy import deepcopy
from typing import List, Any, Optional, TypeVar

import pymongo
from pip_services3_commons.config import ConfigParams
from pip_services3_commons.config import IConfigurable
from pip_services3_commons.data import PagingParams, DataPage
from pip_services3_commons.errors import ConnectionException, InvalidStateException
from pip_services3_commons.refer import IReferenceable, DependencyResolver, IReferences, IUnreferenceable
from pip_services3_commons.reflect import PropertyReflector
from pip_services3_commons.run import IOpenable, ICleanable
from pip_services3_components.log import CompositeLogger
from pymongo.collection import Collection

from pip_services3_mongodb.connect.MongoDbConnection import MongoDbConnection
from .MongoDbIndex import MongoDbIndex
from ..connect.MongoDbConnectionResolver import MongoDbConnectionResolver

filtered = filter

T = TypeVar('T')  # Declare type variable


class MongoDbPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpenable, ICleanable):
    """
    Abstract persistence component that stores data in MongoDB
    using the official MongoDB driver.

    This is the most basic persistence component that is only
    able to store data items of any type. Specific CRUD operations
    over the data items must be implemented in child classes by
    accessing **self.__collection** or **self.__model** properties.

    ### Configuration parameters ###
        - collection:                  (optional) MongoDB collection name
        - connection(s):
            - discovery_key:             (optional) a key to retrieve the connection from :class:`IDiscovery <pip_services3_components.connect.IDiscovery.IDiscovery>`
            - host:                      host name or IP address
            - port:                      port number (default: 27017)
            - uri:                       resource URI or connection string with all parameters in it
        - credential(s):
            - store_key:                 (optional) a key to retrieve the credentials from :class:`ICredentialStore <pip_services3_components.auth.ICredentialStore.ICredentialStore>`
            - username:                  (optional) user name
            - password:                  (optional) user password
        - options:
            - max_pool_size:             (optional) maximum connection pool size (default: 2)
            - keep_alive:                (optional) enable connection keep alive (default: true)
            - connect_timeout:           (optional) connection timeout in milliseconds (default: 5000)
            - socket_timeout:            (optional) socket timeout in milliseconds (default: 360000)
            - auto_reconnect:            (optional) enable auto reconnection (default: true)
            - reconnect_interval:        (optional) reconnection interval in milliseconds (default: 1000)
            - max_page_size:             (optional) maximum page size (default: 100)
            - replica_set:               (optional) name of replica set
            - ssl:                       (optional) enable SSL connection (default: false)
            - auth_source:               (optional) authentication source
            - debug:                     (optional) enable debug output (default: false).

    ### References ###
        - `*:logger:*:*:1.0`           (optional) :class:`ILogger <pip_services3_components.log.ILogger.ILogger>` components to pass log messages
        - `*:discovery:*:*:1.0`        (optional) :class:`IDiscovery <pip_services3_components.connect.IDiscovery.IDiscovery>` services
        - `*:credential-store:*:*:1.0` (optional) :class:`ICredentialStore <pip_services3_components.auth.ICredentialStore.ICredentialStore>` stores to resolve credentials

    Example:

    .. code-block:: python

        class MyMongoDbPersistence(MongoDbPersistence):
            def __init__(self):
                super(MyMongoDbPersistence, self).__init__("mydata", MyData)

            def get_by_name(self, correlationId, name):
                item =  self._collection.find_one({ 'name': name })
                return item

            def set(self, correlationId, item):
                item = self._collection.find_one_and_update(
                    { '_id': item.id }, { '$set': item },
                    return_document = pymongo.ReturnDocument.AFTER,
                    upsert = True
                    )

        persistence = MyMongoDbPersistence()
        persistence.configure(ConfigParams.from_tuples("host", "localhost", "port", 27017))

        persitence.open("123")

        persistence.set("123", { name: "ABC" })
        item = persistence.get_by_name("123", "ABC")

        print (item)
    """
    __default_config = ConfigParams.from_tuples(
        "collection", None,
        "dependencies.connection", "*:connection:mongodb:*:1.0",

        # "connect.type", "mongodb",
        # "connect.database", "test",
        # "connect.host", "localhost",
        # "connect.port", 27017,

        "options.max_pool_size", 2,
        "options.keep_alive", 1,
        "options.connect_timeout", 5000,
        "options.auto_reconnect", True,
        "options.max_page_size", 100,
        "options.debug", True
    )

    def __init__(self, collection: str = None):
        """
        Creates a new instance of the persistence component.

        :param collection: (optional) a collection name.
        """
        self._lock: threading.Lock = threading.Lock()
        self._connection_resolver: MongoDbConnectionResolver = MongoDbConnectionResolver()
        self._options: ConfigParams = ConfigParams()

        # The logger.
        self._logger: CompositeLogger = CompositeLogger()

        # The dependency resolver.
        self._dependency_resolver = DependencyResolver(self.__default_config)

        # The MongoDB database name.
        self._database_name: str = None
        # The MongoDb database object.
        self._db: Any = None
        # The MongoDb collection object.
        self._collection: Collection = None
        # The MongoDB connection object.
        self._client: Any = None
        # The MongoDB connection component.
        self._connection: MongoDbConnection = None

        self._max_page_size = 100

        # The MongoDB colleciton object.
        self._collection_name: str = collection

        self.__config: ConfigParams = None
        self.__references: IReferences = None
        self.__opened = False
        self.__local_connection = False
        self.__indexes: List[MongoDbIndex] = []

    def configure(self, config: ConfigParams):
        """
        Configures component by passing configuration parameters.

        :param config: configuration parameters to be set.
        """
        config = config.set_defaults(self.__default_config)
        self.__config = config

        self._logger.configure(config)
        self._connection_resolver.configure(config)
        self._dependency_resolver.configure(config)

        self._max_page_size = config.get_as_integer_with_default("options.max_page_size", self._max_page_size)
        self._collection_name = config.get_as_string_with_default('collection', self._collection_name)

    def set_references(self, references: IReferences):
        """
        Sets references to dependent components.

        :param references: references to locate the component dependencies.
        """
        self.__references = references
        self._logger.set_references(references)
        self._connection_resolver.set_references(references)

        # Get connection
        self._dependency_resolver.set_references(references)
        self._connection = self._dependency_resolver.get_one_optional('connection')
        # Or create a local one
        if self._connection is None:
            self._connection = self.__create_connection()
            self.__local_connection = True
        else:
            self.__local_connection = False

    def unset_references(self):
        """
        Unsets (clears) previously set references to dependent components.
        """
        self._connection = None

    def __create_connection(self) -> MongoDbConnection:
        connection = MongoDbConnection()

        if self.__config:
            connection.configure(self.__config)

        if self.__references:
            connection.set_references(self.__references)

        return connection

    def _ensure_index(self, keys: Any, options: Any = None):
        """
        Adds index definition to create it on opening

        :param keys: index keys (fields)
        :param options: index options
        """
        if not keys:
            return
        self.__indexes.append(MongoDbIndex(keys, options))

    def _clear_schema(self):
        """
        Clears all auto-created objects
        """
        self.__indexes = []

    def _define_schema(self):
        # TODO: override in child class
        pass

    def _convert_to_public(self, value: Any) -> Any:
        """
        Converts object value from internal to public format.

        :param value: an object in internal format to convert.

        :return: converted object in public format.
        """
        if value is None: return None
        if '_id' in value.keys():
            value['id'] = value['_id']
            value.pop('_id', None)

        return type('object', (object,), value)

    def _convert_from_public(self, value: Any) -> Any:
        """
        Convert object value from public to internal format.

        :param value: an object in public format to convert.

        :return: converted object in internal format.
        """
        if isinstance(value, dict):
            return deepcopy(value)

        value = PropertyReflector.get_properties(value)

        if 'id' in value.keys():
            value['_id'] = value.get('id') or value.get('_id')
            value.pop('id', None)
        return value

    def is_open(self) -> bool:
        """
        Checks if the component is opened.

        :return: true if the component has been opened and false otherwise.
        """
        return self.__opened

    def open(self, correlation_id: Optional[str]):
        """
        Opens the component.

        :param correlation_id: (optional) transaction id to trace execution through call chain.
        """
        if self.__opened:
            return

        if self._connection is None:
            self._connection = self.__create_connection()
            self.__local_connection = True

        if self.__local_connection:
            self._connection.open(correlation_id)

        if self._connection is None:
            raise InvalidStateException(correlation_id, 'NO_CONNECTION', 'MongoDB connection is missing')

        if not self._connection.is_open():
            raise ConnectionException(correlation_id, "CONNECT_FAILED", "MongoDB connection is not opened")

        self.__opened = False

        self._client = self._connection.get_connection()
        self._db = self._connection.get_database()
        self._database_name = self._connection.get_database_name()

        try:
            self._collection = self._db.get_collection(self._collection_name)

            # Define database schema
            self._define_schema()

            # Recreate indexes
            for index in self.__indexes:
                keys = [(k, pymongo.ASCENDING) if v > 0 else (k, pymongo.DESCENDING) for k, v in index.keys.items()]
                index.options = index.options or {}

                self._collection.create_index(keys, **(index.options or {}))

                index_name = index.options.get('name') or ','.join(deepcopy(index.keys))
                self._logger.debug(correlation_id, "Created index %s for collection %s", index_name,
                                   self._collection_name)

            self.__opened = True
            self._logger.debug(correlation_id, "Connected to mongodb database %s, collection %s", self._database_name,
                               self._collection_name)
        except Exception as ex:
            raise ConnectionException(correlation_id, "CONNECT_FAILED", "Connection to mongodb failed").with_cause(ex)

    def __del_none_objects(self, settings):
        new_settings = {}
        for k in settings.keys():
            if settings[k] is not None:
                new_settings[k] = settings[k]
        return new_settings

    def close(self, correlation_id: Optional[str]):
        """
        Closes component and frees used resources.

        :param correlation_id: (optional) transaction id to trace execution through call chain.
        """
        if not self.__opened:
            return

        try:
            if self._client is not None:
                self._client.close()

            if self._connection is None:
                raise InvalidStateException(correlation_id, 'NO_CONNECTION', 'MongoDb connection is missing')

            if self.__local_connection:
                self._connection.close(correlation_id)

            self._collection = None
            self._db = None
            self._client = None

            self.__opened = False
            self._logger.debug(correlation_id, "Disconnected from mongodb database " + str(self._database_name))
        except Exception as ex:
            raise ConnectionException(None, 'DISCONNECT_FAILED', 'Disconnect from mongodb failed: ' + str(ex)) \
                .with_cause(ex)

    def clear(self, correlation_id: Optional[str]):
        """
        Clears component state.

        :param correlation_id: (optional) transaction id to trace execution through call chain.
        """
        if self._collection_name is None:
            raise Exception("Collection name is not defined")

        self._collection.delete_many({})

    def create(self, correlation_id: Optional[str], item: T) -> T:
        """
        Creates a data item.

        :param correlation_id: (optional) transaction id to trace execution through call chain.

        :param item: an item to be created.

        :return: a created item
        """
        if item is None:
            return

        new_item = self._convert_from_public(item)

        result = self._collection.insert_one(new_item)
        item = self._collection.find_one({'_id': result.inserted_id})

        item = self._convert_to_public(item)
        return item

    def delete_by_filter(self, correlation_id: Optional[str], filter: Any):
        """
        Deletes data items that match to a given filter.

        This method shall be called by a public :func:`delete_by_filter` method from child class that
        receives :class:`FilterParams <pip_services3_commons.data.FilterParams.FilterParams>` and converts them into a filter function.

        :param correlation_id: (optional) transaction id to trace execution through call chain.

        :param filter: (optional) a filter function to filter items.
        """
        result = self._collection.delete_many(filter or {})
        count = 0 if result is None else result.deleted_count
        self._logger.trace(correlation_id, "Deleted %d items from %s", count, self._collection_name)

    def get_one_random(self, correlation_id: Optional[str], filter: Any) -> Optional[T]:
        """
        Gets a random item from items that match to a given filter.

        This method shall be called by a public get_one_random method from child class
        that receives FilterParams and converts them into a filter function.

        :param correlation_id: (optional) transaction id to trace execution through call chain.

        :return: a random item.
        """
        count = self._collection.count_documents(filter or {})

        pos = random.randint(0, count)

        statement = self._collection.find(filter).skip(pos if pos > 0 else 0).limit(1)

        for item in statement:
            if item is None:
                self._logger.trace(correlation_id, "Random item wasn't found from %s", self._collection_name)
            else:
                self._logger.trace(correlation_id, "Retrieved random item from %s", self._collection_name)

            item = self._convert_to_public(item)

            return item

        return None

    def get_page_by_filter(self, correlation_id: Optional[str], filter: Any, paging: PagingParams,
                           sort: Any = None, select: Any = None) -> DataPage:
        """
        Gets a page of data items retrieved by a given filter and sorted according to sort parameters.

        This method shall be called by a public get_page_by_filter method from child class that
        receives FilterParams and converts them into a filter function.

        :param correlation_id: (optional) transaction id to trace execution through call chain.
        :param filter: (optional) a filter JSON object
        :param paging: (optional) paging parameters
        :param sort: (optional) sorting JSON object
        :param select: (optional) projection JSON object
        :return: a data page of result by filter
        """
        # Adjust max item count based on configuration
        paging = paging if paging is not None else PagingParams()
        skip = paging.get_skip(-1)
        take = paging.get_take(self._max_page_size)
        paging_enabled = paging.total
        filter = filter or {}

        # Configure statement
        statement = self._collection.find(filter, projection=select or {})

        if skip >= 0:
            statement = statement.skip(skip)
        statement = statement.limit(take)
        if sort is not None:
            statement = statement.sort(sort)

        # Retrive page items
        items = []
        for item in statement:
            item = self._convert_to_public(item)
            items.append(item)

        if items:
            self._logger.trace(correlation_id, "Retrieved %d from %s", len(items), self._collection_name)

        # Calculate total if needed
        total = None
        if paging_enabled:
            total = self._collection.count_documents(filter)

        return DataPage(items, total)

    def get_list_by_filter(self, correlation_id: Optional[str], filter: Any,
                           sort: Any = None, select: Any = None) -> List[T]:
        """
        Gets a list of data items retrieved by a given filter and sorted according to sort parameters.

        This method shall be called by a public get_list_by_filter method from child class that
        receives FilterParams and converts them into a filter function.

        :param correlation_id: (optional) transaction id to trace execution through call chain.

        :param filter: (optional) a filter function to filter items

        :param sort: (optional) sorting parameters

        :param select: (optional) projection parameters (not used yet)

        :return: a data list of results by filter.
        """
        # Configure statement
        filter = filter or {}
        statement = self._collection.find(filter, projection=select or {})

        if sort is not None:
            statement = statement.sort(sort)

        # Retrive page items
        items = []
        for item in statement:
            item = self._convert_to_public(item)
            items.append(item)

        if items:
            self._logger.trace(correlation_id, "Retrieved %d from %s", len(items), self._collection_name)

        return items

    def get_count_by_filter(self, correlation_id: Optional[str], filter: Any) -> int:
        """
        Gets a number of data items retrieved by a given filter.

        This method shall be called by a public get_count_by_filter method from child class that
        receives FilterParams and converts them into a filter function.

        :param correlation_id: (optional) transaction id to trace execution through call chain.
        :param filter: (optional) a filter JSON object
        :return: a number of filtered items.
        """
        filter = filter or {}
        count = self._collection.count_documents(filter)

        if count is not None:
            self._logger.trace(correlation_id, "Counted %d items in %s", count, self._collection_name)

        return count



# -*- coding: utf-8 -*-
"""
    pip_services3_mongodb.__init__
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Mongodb module initialization
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

__all__ = [ ]





# -*- coding: utf-8 -*-
"""
    tests.persistence.test_DummyMongoDbPersistence
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    :copyright: (c) Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

import os

from pip_services3_commons.config import ConfigParams

from .DummyMongoDbPersistence import DummyMongoDbPersistence
from test.fixtures.DummyPersistenceFixture import DummyPersistenceFixture


class TestDummyMongoDbPersistence:
    persistence = None
    fixture = None

    mongoUri = os.getenv('MONGO_URI')
    mongoHost = os.getenv('MONGO_HOST') if os.getenv('MONGO_HOST') else 'localhost'
    mongoPort = os.getenv('MONGO_PORT') if os.getenv('MONGO_PORT') else 27017
    mongoDatabase = os.getenv('MONGO_DB') if os.getenv('MONGO_DB') else 'test'

    @classmethod
    def setup_class(cls):
        if cls.mongoUri is None and cls.mongoHost is None:
            return

        db_config = ConfigParams.from_tuples('connection.uri', cls.mongoUri,
                                             'connection.host', cls.mongoHost,
                                             'connection.port', cls.mongoPort,
                                             'connection.database', cls.mongoDatabase)
        cls.persistence = DummyMongoDbPersistence()
        cls.fixture = DummyPersistenceFixture(cls.persistence)

        cls.persistence.configure(db_config)
        cls.persistence.open(None)

    @classmethod
    def teardown_class(cls):
        cls.persistence.close(None)

    def setup_method(self, method):
        self.persistence.clear(None)

    def test_crud_operations(self):
        self.fixture.test_crud_operations()

    def test_batch_operations(self):
        self.fixture.test_batch_operations()



# -*- coding: utf-8 -*-
"""
    pip_services3_mongodb.connect.MongoDbConnectionResolver
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    MongoDbConnectionResolver implementation.

    :copyright: Conceptual Vision Consulting LLC 2018-2019, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""
from typing import Optional, List

from pip_services3_commons.config import IConfigurable, ConfigParams
from pip_services3_commons.errors import ConfigException
from pip_services3_commons.refer import IReferenceable, IReferences
from pip_services3_components.auth import CredentialResolver, CredentialParams
from pip_services3_components.connect import ConnectionResolver, ConnectionParams


class MongoDbConnectionResolver(IReferenceable, IConfigurable):
    """
    MongoDbConnectionResolver implementation.
    Helper class that resolves MongoDB connection
    and credential parameters, validates them and generates a connection URI.

    It is able to process multiple connections to MongoDB cluster nodes.

    ### Configuration parameters ###
        - connection(s):
            - discovery_key:               (optional) a key to retrieve the connection from IDiscovery
            - host:                        host name or IP address
            - port:                        port number (default: 27017)
            - database:                    database name
            - uri:                         resource URI or connection string with all parameters in it
        - credential(s):
            - store_key:                   (optional) a key to retrieve the credentials from ICredentialStore
            - username:                    user name
            - password:                    user password

    ### References ###
        - `*:discovery:*:*:1.0`             (optional) :class:`IDiscovery <pip_services3_components.connect.IDiscovery.IDiscovery>` services
        - `*:credential-store:*:*:1.0`      (optional) :class:`ICredentialStore <pip_services3_components.auth.ICredentialStore.ICredentialStore>` stores to resolve credentials
    """

    def __init__(self):
        self._connection_resolver: ConnectionResolver = ConnectionResolver()
        self._credential_resolver: CredentialResolver = CredentialResolver()

    def configure(self, config: ConfigParams):
        """
        Configures component by passing configuration parameters.

        :param config: configuration parameters to be set.
        """
        self._connection_resolver.configure(config)
        self._credential_resolver.configure(config)

    def set_references(self, references: IReferences):
        """
        Sets references to dependent components.

        :param references: references to locate the component dependencies.
        """
        self._connection_resolver.set_references(references)
        self._credential_resolver.set_references(references)

    def __validate_connection(self, correlation_id: Optional[str], connection: ConnectionParams):
        uri = connection.get_uri()
        if uri is None:
            return None

        host = connection.get_host()
        if host is None:
            return ConfigException(correlation_id, "NO_HOST", "Connection host is not set")

        port = connection.get_port()
        if port == 0:
            return ConfigException(correlation_id, "NO_PORT", "Connection port is not set")

        database = connection.get_as_nullable_string("database")
        if database is None:
            return ConfigException(correlation_id, "NO_DATABASE", "Connection database is not set")

    def __validate_connections(self, correlation_id: Optional[str], connections: List[ConnectionParams]):
        if connections is None or len(connections) == 0:
            return ConfigException(correlation_id, "NO_CONNECTION", "Database connection is not set")

        for connection in connections:
            error = self.__validate_connection(correlation_id, connection)

    def __compose_uri(self, connections: List[ConnectionParams], credential: CredentialParams) -> str:
        for connection in connections:
            uri = connection.get_uri()
            if uri:
                return uri

        hosts = ''
        for connection in connections:
            host = connection.get_host()
            port = connection.get_port()

            if len(hosts) > 0:
                hosts = hosts + ','
            hosts = hosts + host + (':' + str(port) if port is not None else '')

        database = ''
        for connection in connections:
            database = connection.get_as_nullable_string("database") \
                if connection.get_as_nullable_string("database") is not None \
                else database

            if len(database) > 0:
                database = '/' + database

        auth = ''
        if credential is not None:
            username = credential.get_username()
            if username is not None:
                password = credential.get_password()
                if password is not None:
                    auth = username + ':' + password + '@'
                else:
                    auth = username + '@'

        options = ConfigParams()
        for connection in connections:
            options = options.override(connection)
        if not (credential is None):
            options = options.override(credential)

        options.remove("uri")
        options.remove("host")
        options.remove("port")
        options.remove("database")
        options.remove("username")
        options.remove("password")

        parameters = ''
        keys = options.get_keys()
        for key in keys:
            if len(parameters) > 0:
                parameters += '&'

            parameters += key

            value = options.get_as_string(key)
            if value is not None:
                parameters += '=' + value

        if len(parameters) > 0:
            parameters = '?' + parameters

        uri = "mongodb://" + auth + hosts + database + parameters

        return uri

    def resolve(self, correlation_id: Optional[str]) -> str:
        """
        Resolves MongoDB connection URI from connection and credential parameters.

        :param correlation_id: (optional) transaction id to trace execution through call chain.

        :return: a resolved URI
        """
        connections = self._connection_resolver.resolve_all(correlation_id)
        credential = self._credential_resolver.lookup(correlation_id)

        self.__validate_connections(correlation_id, connections)

        return self.__compose_uri(connections, credential)



# -*- coding: utf-8 -*-
from typing import Any, Optional

import pymongo
from pip_services3_commons.config import ConfigParams
from pip_services3_commons.errors.ConnectionException import ConnectionException
from pip_services3_commons.refer import IReferenceable, IReferences
from pip_services3_commons.run.IOpenable import IOpenable
from pip_services3_components.log.CompositeLogger import CompositeLogger
from pymongo import database

from pip_services3_mongodb.connect.MongoDbConnectionResolver import MongoDbConnectionResolver


class MongoDbConnection(IReferenceable, IReferences, IOpenable):
    """
    MongoDB connection using plain driver.

    By defining a connection and sharing it through multiple persistence components
    you can reduce number of used database connections.

    ### Configuration parameters ###
        - connection(s):
          - discovery_key:             (optional) a key to retrieve the connection from :class:`IDiscovery <pip_services3_components.connect.IDiscovery.IDiscovery>`
          - host:                      host name or IP address
          - port:                      port number (default: 27017)
          - uri:                       resource URI or connection string with all parameters in it
        - credential(s):
          - store_key:                 (optional) a key to retrieve the credentials from :class:`ICredentialStore <pip_services3_components.auth.ICredentialStore.ICredentialStore>`
          - username:                  (optional) user name
          - password:                  (optional) user password
        - options:
          - max_pool_size:             (optional) maximum connection pool size (default: 2)
          - connect_timeout:           (optional) connection timeout in milliseconds (default: 5000)
          - socket_timeout:            (optional) socket timeout in milliseconds (default: 360000)
          - auto_reconnect:            (optional) enable auto reconnection (default: true)
          - reconnect_interval:        (optional) reconnection interval in milliseconds (default: 1000)
          - max_page_size:             (optional) maximum page size (default: 100)
          - replica_set:               (optional) name of replica set
          - ssl:                       (optional) enable SSL connection (default: false)
          - auth_source:               (optional) authentication source
          - debug:                     (optional) enable debug output (default: false).

    ### References ###
        - **\*:logger:\*:\*:1.0**           (optional)  :class:`ILogger <pip_services3_components.log.ILogger.ILogger>` components to pass log messages
        - **\*:discovery:\*:\*:1.0**        (optional)  :class:`IDiscovery <pip_services3_components.connect.IDiscovery.IDiscovery>` services
        - **\*:credential-store:\*:\*:1.0** (optional)  :class:`ICredentialStore <pip_services3_components.auth.ICredentialStore.ICredentialStore>` stores to resolve credentials
    """

    def __init__(self):
        """
        Creates a new instance of the connection component.
        """
        self.__default_config = ConfigParams.from_tuples(
            'options.max_pool_size', 2,
            'options.connect_timeout', 5000,
            'options.auto_reconnect', True,
            'options.max_page_size', 100,
            'options.debug', True
        )

        # The logger
        self._logger: CompositeLogger = CompositeLogger()
        # The connection resolver
        self._connection_resolver: MongoDbConnectionResolver = MongoDbConnectionResolver()
        # The configuration options.
        self._options: ConfigParams = ConfigParams()
        # The MongoDB connection object.
        self._connection: pymongo.MongoClient = None
        # The MongoDB database name.
        self._database_name: str = None
        # The MongoDb database object.
        self._db: database.Database = None

    def configure(self, config: ConfigParams):
        """
        Configures component by passing configuration parameters.

        :param config: configuration parameters to be set.
        """
        config = config.set_defaults(self.__default_config)
        self._connection_resolver.configure(config)
        self._options = self._options.override(config.get_section('options'))

    def set_references(self, references: IReferences):
        """
        Sets references to dependent components.

        :param references:  references to locate the component dependencies.
        """
        self._logger.set_references(references)
        self._connection_resolver.set_references(references)

    def is_open(self) -> bool:
        """
        Checks if the component is opened.

        :return: true if the component has been opened and false otherwise.
        """
        return self._connection is not None

    def __compose_settings(self) -> Any:
        max_pool_size = self._options.get_as_nullable_string('max_pool_size')
        connection_timeout_ms = self._options.get_as_nullable_integer('connect_timeout')
        socket_timeout_ms = self._options.get_as_nullable_integer('socket_timeout')
        auto_reconnect = self._options.get_as_nullable_boolean('auto_reconnect')
        reconnect_interval = self._options.get_as_nullable_integer('reconnect_interval')
        debug = self._options.get_as_nullable_boolean('debug')

        ssl = self._options.get_as_nullable_boolean('ssl')
        replica_set = self._options.get_as_nullable_string('replica_set')
        auth_source = self._options.get_as_nullable_string('auth_source')
        auth_user = self._options.get_as_nullable_string('auth_user')
        auth_password = self._options.get_as_nullable_string('auth_password')

        settings = {
            'maxPoolSize': max_pool_size,
            'connectTimeoutMS': connection_timeout_ms,
            'socketTimeoutMS': socket_timeout_ms,
        }

        if ssl is not None:
            settings['ssl'] = ssl
        if replica_set is not None:
            settings['replica_set'] = replica_set
        if auth_source is not None:
            settings['auth_source'] = auth_source
        if auth_user is not None:
            settings['auth.user'] = auth_user
        if auth_password is not None:
            settings['auth.password'] = auth_password

        return settings

    def open(self, correlation_id: Optional[str]):
        """
        Opens the component.

        :param correlation_id: (optional) transaction id to trace execution through call chain.
        """
        self._logger.debug(correlation_id, 'Connecting to mongodb')

        try:
            settings = self.__compose_settings()

            # settings['use_new_url_parser'] = True
            # settings['use_undefined_topology'] = True
            settings['appname'] = correlation_id

            uri = self._connection_resolver.resolve(correlation_id)
            settings = self.__del_none_objects(settings)
            client = pymongo.MongoClient(uri, **settings)
            self._connection = client
            self._db = client.get_database()
            self._database_name = self._db.name
        except Exception as ex:
            raise ConnectionException(correlation_id, 'CONNECT_FAILED', 'Connection to mongodb failed').with_cause(ex)

    def __del_none_objects(self, settings: dict):
        new_settings = {}
        for k in settings.keys():
            if settings[k] is not None:
                new_settings[k] = settings[k]
        return new_settings

    def close(self, correlation_id: Optional[str]):
        """
        Closes component and frees used resources.

        :param correlation_id: (optional) transaction id to trace execution through call chain.
        """
        if self._connection is None:
            return

        try:
            self._connection.close()
            self._connection = None
            self._db = None
            self._database_name = None
            self._logger.debug(correlation_id, 'Disconnected from mongodb database {}'.format(self._database_name))
        except Exception as ex:
            raise ConnectionException(correlation_id, 'DISCONNECT_FAILED',
                                      'Disconnect from mongodb failed: ').with_cause(ex)

    def get_connection(self) -> pymongo.MongoClient:
        return self._connection

    def get_database(self) -> database.Database:
        return self._db

    def get_database_name(self) -> str:
        return self._database_name



# -*- coding: utf-8 -*-
"""
    pip_services3_mongodb.persistence.IdentifiableMongoDbPersistence
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Identifiable MongoDb persistence implementation
    
    :copyright: Conceptual Vision Consulting LLC 2018-2019, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""
from copy import deepcopy
from typing import Any, Optional, List, TypeVar

import pymongo
from pip_services3_commons.data import AnyValueMap, IdGenerator

from .MongoDbPersistence import MongoDbPersistence

T = TypeVar('T')  # Declare type variable


class IdentifiableMongoDbPersistence(MongoDbPersistence):
    """
    Abstract persistence component that stores data in MongoDB
    and implements a number of CRUD operations over data items with unique ids.
    The data items must implement IIdentifiable interface.

    In basic scenarios child classes shall only override :func:`get_page_by_filter <pip_services3_mongodb.persistence.MongoDbPersistence.get_page_by_filter>`,
    :func:`get_list_by_filter` or :func:`delete_by_filter` operations with specific filter function.
    All other operations can be used out of the box.

    In complex scenarios child classes can implement additional operations by
    accessing **self.__collection** and **self.__model** properties.

    ### Configuration parameters ###

        - connection(s):
            - discovery_key:             (optional) a key to retrieve the connection from :class:`IDiscovery <pip_services3_components.connect.IDiscovery.IDiscovery>`
            - host:                      host name or IP address
            - port:                      port number (default: 27017)
            - uri:                       resource URI or connection string with all parameters in it
        - credential(s):
            - store_key:                 (optional) a key to retrieve the credentials from :class:`ICredentialStore <pip_services3_components.auth.ICredentialStore.ICredentialStore>`
            - username:                  (optional) user name
            - password:                  (optional) user password
        - options:
            - max_pool_size:             (optional) maximum connection pool size (default: 2)
            - keep_alive:                (optional) enable connection keep alive (default: true)
            - connect_timeout:           (optional) connection timeout in milliseconds (default: 5000)
            - socket_timeout:            (optional) socket timeout in milliseconds (default: 360000)
            - auto_reconnect:            (optional) enable auto reconnection (default: true)
            - reconnect_interval:        (optional) reconnection interval in milliseconds (default: 1000)
            - max_page_size:             (optional) maximum page size (default: 100)
            - replica_set:               (optional) name of replica set
            - ssl:                       (optional) enable SSL connection (default: false)
            - auth_source:               (optional) authentication source
            - debug:                     (optional) enable debug output (default: false).

    ### References ###
        - `*:logger:*:*:1.0`           (optional) :class:`ILogger <pip_services3_components.log.ILogger.ILogger>` components to pass log messages components to pass log messages
        - `*:discovery:*:*:1.0`        (optional) :class:`IDiscovery <pip_services3_components.connect.IDiscovery.IDiscovery>` services
        - `*:credential-store:*:*:1.0` (optional) :class:`ICredentialStore <pip_services3_components.auth.ICredentialStore.ICredentialStore>` stores to resolve credentials

    Example:

    .. code-block:: python
    
        class MyMongoDbPersistence(MongoDbPersistence):
            def __init__(self):
                super(MyMongoDbPersistence, self).__init__("mydata", MyData)

            def get_page_by_filter(self, correlation_id, filter, paging, sort = None, select = None):
                super().def get_page_by_filter(correlation_id, filter, paging, None, None):

        persistence = MyMongoDbPersistence()
        persistence.configure(ConfigParams.from_tuples("host", "localhost", "port", 27017))

        persitence.open("123")
        persistence.create("123", { id: "1", name: "ABC" })
        mydata = persistence.get_page_by_filter("123", FilterParams.from_tuples("name", "ABC"), None, None)

        print(mydata)

        persistence.delete_by_id("123", "1")
        # ...
    """

    def __init__(self, collection: str = None):
        """
        Creates a new instance of the persistence component.

        :param collection: (optional) a collection name.
        """
        super(IdentifiableMongoDbPersistence, self).__init__(collection)

        # Flag to turn on automated string ID generation
        self._auto_generate_id: bool = True

    def _convert_from_public_partial(self, value: Any) -> Any:
        """
        Converts the given object from the public partial format.

        :param value: the object to convert from the public partial format.
        :return: the initial object.
        """
        return self._convert_from_public(value)

    def get_list_by_ids(self, correlation_id: Optional[str], ids: List[Any]) -> List[T]:
        """
        Gets a list of data items retrieved by given unique ids.

        :param correlation_id: (optional) transaction id to trace execution through call chain.

        :param ids: ids of data items to be retrieved

        :return: a data list of results by ids.
        """
        filters = {'_id': {'$in': ids}}
        return self.get_list_by_filter(correlation_id, filters)

    def get_one_by_id(self, correlation_id: Optional[str], id: Any) -> T:
        """
        Gets a data item by its unique id.

        :param correlation_id: (optional) transaction id to trace execution through call chain.

        :param id: an id of data item to be retrieved.

        :return: data item by id.
        """
        item = self._collection.find_one({'_id': id})
        if item:
            self._logger.trace(correlation_id, "Nothing found from %s with id = %s", self._collection_name, id)
        else:
            self._logger.trace(correlation_id, "Retrieved from %s with id = %s", self._collection_name, id)

        item = self._convert_to_public(item)
        return item

    def create(self, correlation_id: Optional[str], item: T) -> T:
        """
        Creates a data item.

        :param correlation_id: (optional) transaction id to trace execution through call chain.

        :param item: an item to be created.

        :return: a created item
        """
        if item is None:
            return

        item = self._convert_from_public(item)
        new_item = deepcopy(item)

        # Replace _id or generate a new one
        if new_item.get('_id') is None and self._auto_generate_id:
            new_item['_id'] = IdGenerator.next_long()

        return super().create(correlation_id, new_item)

    def set(self, correlation_id: Optional[str], item: T) -> T:
        """
        Sets a data item. If the data item exists it updates it, otherwise it create a new data item.

        :param correlation_id: (optional) transaction id to trace execution through call chain.

        :param item: an item to be set.

        :return: an updated item
        """
        if item is None:
            return

        item = self._convert_from_public(item)
        new_item = dict(item)

        # Replace _id or generate a new one
        if new_item.get('_id') is None and self._auto_generate_id:
            new_item['_id'] = IdGenerator.next_long()

        new_item = self._convert_from_public(new_item)

        item = self._collection.find_one_and_replace(
            {'_id': new_item['_id']}, new_item,
            return_document=pymongo.ReturnDocument.AFTER,
            upsert=True
        )

        item = self._convert_to_public(item)

        if item:
            self._logger.trace(correlation_id, "Set in %s with id = %s", self._collection_name, item.id)

        return item

    def update(self, correlation_id: Optional[str], item: T) -> Optional[T]:
        """
        Updates a data item.

        :param correlation_id: (optional) transaction id to trace execution through call chain.

        :param item: an item to be updated.

        :return: an updated item.
        """
        if item is None or item.id is None:
            return

        new_item = deepcopy(item)
        new_item = self._convert_from_public(new_item)

        result = self._collection.find_one_and_update(
            {'_id': item.id}, {'$set': new_item},
            return_document=pymongo.ReturnDocument.AFTER
        )

        new_item = self._convert_to_public(result)

        self._logger.trace(correlation_id, "Updated in %s with id = %s", self._collection_name, item.id)

        return new_item

    def update_partially(self, correlation_id: Optional[str], id: Any, data: AnyValueMap) -> T:
        """
        Updates only few selected fields in a data item.

        :param correlation_id: (optional) transaction id to trace execution through call chain.

        :param id: an id of data item to be updated.

        :param data: a map with fields to be updated.

        :return: an updated item.
        """
        if data is None or id is None:
            return

        new_item = data.get_as_object()
        new_item = self._convert_from_public_partial(new_item)

        item = self._collection.find_one_and_update(
            {'_id': id}, {'$set': new_item},
            return_document=pymongo.ReturnDocument.AFTER
        )

        self._logger.trace(correlation_id, "Updated partially in %s with id = %s", self._collection_name, id)

        item = self._convert_to_public(item)
        return item

    # The method must return deleted value to be able to do clean up like removing references
    def delete_by_id(self, correlation_id: Optional[str], id: Any) -> T:
        """
        Deleted a data item by it's unique id.

        :param correlation_id: (optional) transaction id to trace execution through call chain.

        :param id: an id of the item to be deleted

        :return: a deleted item.
        """
        item = self._collection.find_one_and_delete({'_id': id})

        self._logger.trace(correlation_id, "Deleted from %s with id = %s", self._collection_name, id)

        old_item = self._convert_to_public(item)
        return old_item

    def delete_by_ids(self, correlation_id: Optional[str], ids: List[Any]):
        """
        Deletes multiple data items by their unique ids.

        :param correlation_id: (optional) transaction id to trace execution through call chain.

        :param ids: ids of data items to be deleted.
        """
        filter = {'_id': {'$in': ids}}
        self.delete_by_filter(correlation_id, filter)



# -*- coding: utf-8 -*-
"""
    pip_services3_mongodb.build.DefaultMongoDbFactory
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

"""

from pip_services3_commons.refer.Descriptor import Descriptor
from pip_services3_components.build.Factory import Factory

from pip_services3_mongodb.connect.MongoDbConnection import MongoDbConnection


class DefaultMongoDbFactory(Factory):
    """
    Creates MongoDb components by their descriptors.
    See :class:`Factory <pip_services3_components.build.Factory.Factory>`, :class:`MongoDbConnection <pip_services3_mongodb.persistence.MongoDbConnection.MongoDbConnection>`
    """
    MongoDbConnectionDescriptor: Descriptor = Descriptor("pip-services", "connection", "mongodb", "*", "1.0")

    def __init__(self):
        """
        Create a new instance of the factory.
        """
        super(DefaultMongoDbFactory, self).__init__()
        self.register_as_type(DefaultMongoDbFactory.MongoDbConnectionDescriptor, MongoDbConnection)



# -*- coding: utf-8 -*-

import os

from pip_services3_commons.config import ConfigParams

from pip_services3_mongodb.connect.MongoDbConnection import MongoDbConnection


class TestDbConnection:
    persistence = None
    fixture = None

    connection = None

    mongoUri = os.getenv('MONGO_URI')
    mongoHost = os.getenv('MONGO_HOST') if os.getenv('MONGO_HOST') != None else 'localhost'
    mongoPort = os.getenv('MONGO_PORT') if os.getenv('MONGO_PORT') != None else 27017
    mongoDatabase = os.getenv('MONGO_DB') if os.getenv('MONGO_DB') != None else 'test'

    @classmethod
    def setup_class(cls):
        if cls.mongoUri is None and cls.mongoHost is None:
            return

        db_config = ConfigParams.from_tuples('connection.uri', cls.mongoUri,
                                             'connection.host', cls.mongoHost,
                                             'connection.port', cls.mongoPort,
                                             'connection.database', cls.mongoDatabase)
        cls.connection = MongoDbConnection()
        cls.connection.configure(db_config)
        cls.connection.open(None)

    @classmethod
    def teardown_class(cls):
        cls.connection.close(None)

    def test_open_and_close(self):
        assert hasattr(self.connection.get_connection(), '__iter__')
        assert hasattr(self.connection.get_database(), '__iter__')
        assert type(self.connection.get_database_name()) == str



# -*- coding: utf-8 -*-
"""
    test.Dummy
    ~~~~~~~~~~
    
    Dummy data object
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""
from pip_services3_commons.data import IStringIdentifiable


class Dummy(IStringIdentifiable):
    def __init__(self, id: str = None, key: str = None, content: str = None):
        self.id = id
        self.key = key
        self.content = content



# -*- coding: utf-8 -*-

import os

from pip_services3_commons.config.ConfigParams import ConfigParams
from pip_services3_commons.refer.Descriptor import Descriptor
from pip_services3_commons.refer.References import References

from pip_services3_mongodb.connect.MongoDbConnection import MongoDbConnection
from test.persistence.DummyMongoDbPersistence import DummyMongoDbPersistence
from test.fixtures.DummyPersistenceFixture import DummyPersistenceFixture


class TestDummyMongoDbConnection:
    persistence = None
    fixture = None

    connection = None

    mongoUri = os.getenv('MONGO_URI')
    mongoHost = os.getenv('MONGO_HOST') if os.getenv('MONGO_HOST') != None else 'localhost'
    mongoPort = os.getenv('MONGO_PORT') if os.getenv('MONGO_PORT') != None else 27017
    mongoDatabase = os.getenv('MONGO_DB') if os.getenv('MONGO_DB') != None else 'test'

    def setup_class(cls):
        if cls.mongoUri is None and cls.mongoHost is None:
            return

        db_config = ConfigParams.from_tuples('connection.uri', cls.mongoUri,
                                             'connection.host', cls.mongoHost,
                                             'connection.port', cls.mongoPort,
                                             'connection.database', cls.mongoDatabase)
        cls.connection = MongoDbConnection()
        cls.connection.configure(db_config)
        cls.persistence = DummyMongoDbPersistence()
        cls.persistence.set_references(References.from_tuples(
            Descriptor("pip-services", "connection", "mongodb", "default", "1.0"), cls.connection
        ))
        cls.fixture = DummyPersistenceFixture(cls.persistence)
        cls.connection.open(None)
        cls.persistence.open(None)
        cls.persistence.clear(None)

    @classmethod
    def teardown_class(cls):
        cls.connection.close(None)
        cls.persistence.close(None)

    def test_crud_operations(self):
        self.fixture.test_crud_operations()

    def test_batch_operations(self):
        self.fixture.test_batch_operations()



"""
Pip.Services Mongodb
------------------

Pip.Services is an open-source library of basic microservices.
pip_services3_mongodb provides MongoDb persistence components.

Links
`````

* `website <http://github.com/pip-services/pip-services>`_
* `development version <http://github.com/pip-services3-python/pip-services3-mongodb-python>`

"""

try:
    readme = open('readme.md').read()
except:
    readme = __doc__

from setuptools import find_packages
from setuptools import setup

setup(
    name='pip_services3_mongodb',
    version='3.2.5',
    url='http://github.com/pip-services3-python/pip-services3-mongodb-python',
    license='MIT',
    author='Conceptual Vision Consulting LLC',
    author_email='seroukhov@gmail.com',
    description='MongoDb persistence components for Pip.Services in Python',
    long_description=readme,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=['config', 'data', 'test']),
    include_package_data=True,
    zip_safe=True,
    platforms='any',
    install_requires=[
        'pymongo >= 4.0.1, < 5.0.0',

        'pip_services3_commons >= 3.3.10, < 4.0',
        'pip_services3_components >= 3.5.4, < 4.0',
        'pip_services3_data >= 3.2.3, < 4.0'
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]    
)



# -*- coding: utf-8 -*-
"""
    tests.DummyPersistenceFixture
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    :copyright: (c) Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

from pip_services3_commons.data import AnyValueMap

from .Dummy import Dummy
from .IDummyPersistence import IDummyPersistence

DUMMY1 = Dummy(None, 'Key 1', 'Content 1')
DUMMY2 = Dummy(None, 'Key 2', 'Content 2')


class DummyPersistenceFixture:
    _persistence = None

    def __init__(self, persistence: IDummyPersistence):
        self._persistence: IDummyPersistence = persistence

    def test_crud_operations(self):
        # Create one dummy
        dummy1 = self._persistence.create(None, DUMMY1)

        assert dummy1 is not None
        assert dummy1.id is not None
        assert DUMMY1.key == dummy1.key
        assert DUMMY1.content == dummy1.content

        # Create another dummy
        dummy2 = self._persistence.create(None, DUMMY2)

        assert dummy2 is not None
        assert dummy2.id is not None
        assert DUMMY2.key == dummy2.key
        assert DUMMY2.content == dummy2.content

        # Get all dummies
        dummies = self._persistence.get_page_by_filter(None, None, None)
        assert dummies is not None
        assert 2 == len(dummies.data)

        # Update the dummy
        dummy1.content = "Updated Content 1"
        result = self._persistence.update(
            None,
            dummy1
        )

        assert result is not None
        assert dummy1.id == result.id
        assert dummy1.key == result.key
        assert dummy1.content == result.content

        # Partially update the dummy
        result = self._persistence.update_partially(
            None,
            dummy1.id,
            AnyValueMap.from_tuples(
                'content', 'Partially Updated Content 1'
            )
        )

        assert result is not None
        assert dummy1.id == result.id
        assert dummy1.key == result.key
        assert "Partially Updated Content 1" == result.content

        # Get the dummy by Id
        result = self._persistence.get_one_by_id(None, dummy1.id)
        assert result is not None
        assert result.id == dummy1.id
        assert result.key == dummy1.key
        assert 'Partially Updated Content 1' == result.content

        # Delete the dummy
        self._persistence.delete_by_id(None, dummy1.id)
        assert result is not None
        assert result.id == dummy1.id
        assert result.key == dummy1.key
        assert 'Partially Updated Content 1' == result.content

        # Try to get deleted dummy
        result = self._persistence.get_one_by_id(None, dummy1.id)
        assert result is None

        count = self._persistence.get_count_by_filter(None, None)
        assert count == 1

    def test_batch_operations(self):
        # Create one dummy
        dummy1 = self._persistence.create(None, DUMMY1)
        assert dummy1 is not None
        assert dummy1.id is not None
        assert DUMMY1.key == dummy1.key
        assert DUMMY1.content == dummy1.content

        # Create one dummy
        dummy2 = self._persistence.create(None, DUMMY2)

        assert dummy2 is not None
        assert dummy2.id is not None
        assert DUMMY2.key == dummy2.key
        assert DUMMY2.content == dummy2.content

        # Read batch
        dummies = self._persistence.get_list_by_ids(None, [dummy1.id, dummy2.id])
        assert isinstance(dummies, list)
        assert 2 == len(dummies)

        # Delete batch
        self._persistence.delete_by_ids(None, [dummy1.id, dummy2.id])

        # Read empty batch
        dummies = self._persistence.get_list_by_ids(None, [dummy1.id, dummy2.id])
        assert isinstance(dummies, list)
        assert 0 == len(dummies)
